let
  pkgs = import <nixpkgs> { };

  gridlib = import ../../lib;
  grid-config = pkgs.lib.trivial.importJSON ./config.json;

  # Module with per-grid configuration
  grid-module = {config, ...}: {
    imports = [
      gridlib.base
      # Allow us to remotely trigger updates to this system.
      ../../../nixos/modules/deployment.nix
      # Give it a good SSH configuration.
      ../../../nixos/modules/ssh.nix
    ];
    services.private-storage.sshUsers = import ./public-keys/users.nix;
    networking.domain = grid-config.domain;
    # Convert relative paths to absolute so library code can resolve names
    # correctly.
    grid = {
      publicKeyPath = toString ./. + "/${grid-config.publicKeyPath}";
      privateKeyPath = toString ./. + "/${grid-config.privateKeyPath}";
    };
    # Configure deployment management authorization for all systems in the grid.
    services.private-storage.deployment = {
      authorizedKey = builtins.readFile "${config.grid.publicKeyPath}/deploy_key.pub";
      gridName = "local";
    };
  };

  payments = {
    imports = [
      gridlib.issuer
      (gridlib.hardware-virtual ({ publicIPv4 = "192.168.67.21"; }))
      (gridlib.customize-issuer (grid-config // {
          monitoringvpnIPv4 = "172.23.23.11";
      }))
      grid-module
    ];
  };

  storage1 = {
    imports = [
      gridlib.storage
      (gridlib.hardware-virtual ({ publicIPv4 = "192.168.67.22"; }))
      (gridlib.customize-storage (grid-config // {
        monitoringvpnIPv4 = "172.23.23.12";
        stateVersion = "19.09";
      }))
      grid-module
    ];
  };

  storage2 = {
    imports = [
      gridlib.storage
      (gridlib.hardware-virtual ({ publicIPv4 = "192.168.67.23"; }))
      (gridlib.customize-storage (grid-config // {
        monitoringvpnIPv4 = "172.23.23.13";
        stateVersion = "19.09";
      }))
      grid-module
    ];
  };

  monitoring = {
    imports = [
      gridlib.monitoring
      (gridlib.hardware-virtual ({ publicIPv4 = "192.168.67.24"; }))
      (gridlib.customize-monitoring {
        inherit hostsMap vpnClientIPs nodeExporterTargets paymentExporterTargets;
        inherit (grid-config) letsEncryptAdminEmail;
        googleOAuthClientID = grid-config.monitoringGoogleOAuthClientID;
        monitoringvpnIPv4 = "172.23.23.1";
        stateVersion = "19.09";
      })
      grid-module
    ];
  };

  # TBD: derive these automatically:
  hostsMap = {
    "172.23.23.1"  = [ "monitoring" "monitoring.monitoringvpn" ];
    "172.23.23.11" = [ "payments" "payments.monitoringvpn" ];
    "172.23.23.12" = [ "storage1" "storage1.monitoringvpn" ];
    "172.23.23.13" = [ "storage2" "storage2.monitoringvpn" ];
  };
  vpnClientIPs = [ "172.23.23.11" "172.23.23.12" "172.23.23.13" ];
  nodeExporterTargets = [ "monitoring" "payments" "storage1" "storage2" ];
  paymentExporterTargets = [ "payments" ];

in {
  network = {
    description = "PrivateStorage.io LocalDev Grid";
  };
  inherit payments monitoring storage1 storage2;
}
